@@ -301,7 +301,7 @@ pub fn invoke_method(name: &str, invoke_args: &CalcitItems, call_stack: &CallSta
             // dirty copy...
             Calcit::Fn {
               def_ns, scope, args, body, ..
-            } => runner::run_fn(&method_args, scope, args, body, def_ns.to_owned(), call_stack),
+            } => runner::run_fn(&method_args, scope, args.to_owned(), body, def_ns.to_owned(), call_stack),
             Calcit::Proc(proc) => builtins::handle_proc(proc, &method_args, call_stack),
             Calcit::Syntax(syn, _ns) => Err(CalcitErr::use_msg_stack(
               format!("cannot get syntax here since instance is always evaluated, got: {}", syn),
