@@ -36,7 +36,7 @@ fn modify_ref(path: Arc<str>, v: Calcit, call_stack: &CallStackList) -> Result<(
         def_ns, scope, args, body, ..
       } => {
         let values = TernaryTreeList::from(&[v.to_owned(), prev.to_owned()]);
-        runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
+        runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
       }
       a => {
         return Err(CalcitErr::use_msg_stack(
@@ -66,10 +66,10 @@ pub fn defatom(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, call_
       Ok(Calcit::Ref(path_info.to_owned()))
     }
     (Some(a), Some(b)) => Err(CalcitErr::use_msg_stack(
-      format!("defref expected a symbol and an expression: {} , {}", a, b),
+      format!("defatom expected a symbol and an expression: {} , {}", a, b),
       call_stack,
     )),
-    _ => Err(CalcitErr::use_msg_stack("defref expected 2 nodes", call_stack)),
+    _ => Err(CalcitErr::use_msg_stack("defatom expected 2 nodes", call_stack)),
   }
 }
 
