@@ -12,18 +12,18 @@ export class CalcitTuple {
     this.cachedHash = null;
   }
   get(n: number) {
-    if (n == 0) {
+    if (n === 0) {
       return this.fst;
-    } else if (n == 1) {
+    } else if (n === 1) {
       return this.snd;
     } else {
       throw new Error("Tuple only have 2 elements");
     }
   }
   assoc(n: number, v: CalcitValue) {
-    if (n == 0) {
+    if (n === 0) {
       return new CalcitTuple(v, this.snd);
-    } else if (n == 1) {
+    } else if (n === 1) {
       return new CalcitTuple(this.fst, v);
     } else {
       throw new Error("Tuple only have 2 elements");
